import { GraphQLError } from 'graphql/error';
import { GraphQLSchema, GraphQLFieldResolver, ResponsePath } from 'graphql/type';
import { DocumentNode } from 'graphql/language/ast';
import { QueryReducer } from './QueryReducer';
import { Middleware } from '../middleware/Middleware';
/**
 * Terminology
 *
 * "Definitions" are the generic name for top-level statements in the document.
 * Examples of this include:
 * 1) Operations (such as a query)
 * 2) Fragments
 *
 * "Operations" are a generic name for requests in the document.
 * Examples of this include:
 * 1) query,
 * 2) mutation
 *
 * "Selections" are the definitions that can appear legally and at
 * single level of the query. These include:
 * 1) field references e.g "a"
 * 2) fragment "spreads" e.g. "...c"
 * 3) inline fragment "spreads" e.g. "...on Type { a }"
 */
export interface ExecutionResult {
    data?: {
        [key: string]: mixed;
    } | null;
    errors?: Array<GraphQLError>;
}
export declare type ExecutionConfig = {
    schema: GraphQLSchema;
    document: DocumentNode;
    rootValue?: mixed;
    contextValue?: mixed;
    variableValues?: {
        [key: string]: mixed;
    };
    operationName?: string;
    queryReducers?: Array<QueryReducer<mixed, mixed>>;
    middleware?: Array<Middleware<mixed, mixed, mixed>>;
};
/**
 * Implements the "Evaluating requests" section of the GraphQL specification.
 *
 * Returns a Promise that will eventually be resolved and never rejected.
 *
 * If the arguments to this function do not result in a legal execution context,
 * a GraphQLError will be thrown immediately explaining the invalid input.
 */
export declare function execute(config: ExecutionConfig): Promise<ExecutionResult>;
/**
 * Given a ResponsePath (found in the `path` entry in the information provided
 * as the last argument to a field resolver), return an Array of the path keys.
 */
export declare function responsePathAsArray(path: ResponsePath): Array<string | number>;
/**
 * If a resolve function is not given, then a default resolve behavior is used
 * which takes the property of the source object of the same name as the field
 * and returns it as the result, or if it's a function, returns the result
 * of calling that function while passing along args and context.
 */
export declare const defaultFieldResolver: GraphQLFieldResolver<mixed, mixed>;
