import { ObjectTypeDefinitionNode, InterfaceTypeDefinitionNode, EnumTypeDefinitionNode, UnionTypeDefinitionNode, ScalarTypeDefinitionNode, InputObjectTypeDefinitionNode, FieldDefinitionNode, GraphQLSchema, GraphQLInterfaceType, GraphQLEnumType, GraphQLUnionType, GraphQLScalarType, GraphQLInputObjectType } from 'graphql';
import { GraphQLFieldConfigExt, GraphQLObjectTypeExt } from '../type/object';
import { SchemaFactory } from './SchemaFactory';
/**
 * Middleware is useful for manipulating how the SchemaFactory creates a schema.
 * Usage:
 *  - Provide a class that extends the FactoryMiddleware class to the SchemaFactory.
 *  - This is implemented as a class instead of an interface so that it can default
 *    middleware methods to the identity function.
 */
export declare class FactoryMiddleware {
    /**
     * This is called directly before the schema attempts to build and
     * provides an opportunity for any excess setup.
     * For example, you might add any default types, interfaces, etc.
     * @param _factory The factory instance calling the middleware
     */
    onInit(_factory: SchemaFactory): void;
    /**
     * This is called immediately after getSchema() is called and
     * provides an opportunity for any excess setup.
     * For example, you might add any default types, interfaces, etc.
     * @param _factory The factory instance calling the middleware
     */
    beforeBuild(_factory: SchemaFactory): void;
    /**
     * This is called directly after the schema is built and
     * provides an opportunity for any last second changes.
     * @param schema The GraphQL schema
     */
    afterBuild(schema: GraphQLSchema): GraphQLSchema;
    /**
     * Allows you to wrap an object type field. Useful for wrapping resolvers
     * @param field The field config. This field may already contain a resolver that was pulled
     *        from the factories resolver cache. You can choose to override or compose them.
     */
    wrapObjectField(_factory: SchemaFactory, _definition: ObjectTypeDefinitionNode, _fieldDef: FieldDefinitionNode, field: GraphQLFieldConfigExt<mixed, mixed>): GraphQLFieldConfigExt<mixed, mixed>;
    /**
     * Allows you to wrap an interface type field.
     * @param field The field config. This field may already contain a resolver that was pulled
     *        from the factories resolver cache. You can choose to override or compose them.
     */
    wrapInterfaceField(_factory: SchemaFactory, _definition: InterfaceTypeDefinitionNode, _fieldDef: FieldDefinitionNode, field: GraphQLFieldConfigExt<mixed, mixed>): GraphQLFieldConfigExt<mixed, mixed>;
    /**
     * Executed before the schema is built. This is useful for manipulating the
     * AST nodes before they are converted into GraphQL types. These middleware methods
     * operate on the AST nodes before they are built into full GraphQL types.
     *
     * @param nodeMap And ImmutableJS Map containing the AST definition nodes.
     * @return The augmented nodeMap
     */
    wrapObjectNode(_factory: SchemaFactory, definition: ObjectTypeDefinitionNode): ObjectTypeDefinitionNode;
    /**
     * Wrap an interface or interact with the factory.
     */
    wrapInterfaceNode(_factory: SchemaFactory, definition: InterfaceTypeDefinitionNode): InterfaceTypeDefinitionNode;
    /**
     * Wrap an enum or interact with the factory.
     */
    wrapEnumNode(_factory: SchemaFactory, definition: EnumTypeDefinitionNode): EnumTypeDefinitionNode;
    /**
     * Wrap an union or interact with the factory.
     */
    wrapUnionNode(_factory: SchemaFactory, definition: UnionTypeDefinitionNode): UnionTypeDefinitionNode;
    /**
     * Wrap an scalar or interact with the factory.
     */
    wrapScalarNode(_factory: SchemaFactory, definition: ScalarTypeDefinitionNode): ScalarTypeDefinitionNode;
    /**
     * Wrap an input or interact with the factory.
     */
    wrapInputNode(_factory: SchemaFactory, definition: InputObjectTypeDefinitionNode): InputObjectTypeDefinitionNode;
    /**
     * Executed before the schema is built. This is useful for manipulating the GraphQL types before
     * they are committed to the factory cache.
     *
     * @param nodeMap And ImmutableJS Map containing the AST definition nodes.
     * @return The augmented nodeMap
     */
    wrapObjectType(_factory: SchemaFactory, _definition: ObjectTypeDefinitionNode, object: GraphQLObjectTypeExt): GraphQLObjectTypeExt;
    /**
     * Wrap an interface type or interact with the factory.
     */
    wrapInterfaceType(_factory: SchemaFactory, _definition: InterfaceTypeDefinitionNode, interfce: GraphQLInterfaceType): GraphQLInterfaceType;
    /**
     * Wrap an enum type or interact with the factory.
     */
    wrapEnumType(_factory: SchemaFactory, _definition: EnumTypeDefinitionNode, enm: GraphQLEnumType): GraphQLEnumType;
    /**
     * Wrap an union type or interact with the factory.
     */
    wrapUnionType(_factory: SchemaFactory, _definition: UnionTypeDefinitionNode, union: GraphQLUnionType): GraphQLUnionType;
    /**
     * Wrap an scalar type or interact with the factory.
     */
    wrapScalarType(_factory: SchemaFactory, _definition: ScalarTypeDefinitionNode, sclr: GraphQLScalarType): GraphQLScalarType;
    /**
     * Wrap an input type or interact with the factory.
     */
    wrapInputType(_factory: SchemaFactory, _definition: InputObjectTypeDefinitionNode, inpt: GraphQLInputObjectType): GraphQLInputObjectType;
}
