import { TypeResolverMap, FieldResolverMap, ScalarResolverMap } from '../utilities/ResolverMap';
import { FactoryMiddleware } from './FactoryMiddleware';
import { Map } from 'immutable';
import { GraphQLFieldResolver } from 'graphql';
import { DirectiveNode, TypeNode, SchemaDefinitionNode, TypeDefinitionNode, ScalarTypeDefinitionNode, ObjectTypeDefinitionNode, InputValueDefinitionNode, InterfaceTypeDefinitionNode, UnionTypeDefinitionNode, EnumTypeDefinitionNode, InputObjectTypeDefinitionNode, DirectiveDefinitionNode, FieldDefinitionNode } from 'graphql/language/ast';
import { GraphQLSchema } from 'graphql/type/schema';
import { GraphQLScalarType, GraphQLObjectType, GraphQLInterfaceType, GraphQLUnionType, GraphQLEnumType, GraphQLInputObjectType, GraphQLTypeResolver } from 'graphql/type/definition';
import { GraphQLType, GraphQLNamedType, GraphQLInputType, GraphQLOutputType } from 'graphql/type/definition';
import { GraphQLDirective } from 'graphql/type/directives';
import { GraphQLFieldConfigMapExt } from '../type/object';
import { GraphQLDirectiveValue } from '../type/directives';
export interface SchemaFactoryConfig {
    middleware?: FactoryMiddleware;
    onResolverCollision?: (oldFn: FieldResolverMap, newFn: FieldResolverMap, key: string) => FieldResolverMap;
    onTypeCollision?: (oldType: GraphQLNamedType, newType: GraphQLNamedType, key: string) => GraphQLNamedType;
}
/**
 * A base class that handles managing and building GraphQL schemas
 */
export declare class SchemaFactory {
    protected schemaDef: SchemaDefinitionNode | null;
    protected typeMap: Map<string, GraphQLNamedType>;
    protected nodeMap: Map<string, TypeDefinitionNode>;
    protected resolverMap: Map<string, FieldResolverMap>;
    protected typeResolverMap: Map<string, GraphQLTypeResolver<mixed, mixed>>;
    protected directiveMap: Map<string, GraphQLDirective>;
    protected scalarResolvers: Map<string, ScalarResolverMap<mixed, mixed>>;
    private config;
    private middleware;
    constructor(config?: SchemaFactoryConfig);
    /**
     * Public API
     */
    private buildSchema();
    getSchema(): GraphQLSchema;
    getType(name: string): GraphQLNamedType;
    hasType(name: string): boolean;
    hasDefinition(name: string): boolean;
    /**
     * Getters
     */
    readonly nodes: Array<TypeDefinitionNode>;
    readonly types: Array<GraphQLNamedType>;
    /**
     * Add resolvers to the resolver map. These resolvers will get merged into types
     * when you call getSchema().
     */
    addResolveFunctionsToSchema(resolvers?: TypeResolverMap<mixed, mixed>): SchemaFactory;
    /**
     * Add a single object type to the schema.
     * @param spec A single type declaration as GraphQL schema IDL
     * @param resolvers An object with field names for keys and GraphQLFieldResolver functions as values.
     */
    createType(spec: string, resolvers?: FieldResolverMap): SchemaFactory;
    /**
     * Add a single interface type to the schema.
     * @param spec A single type declaration as GraphQL schema IDL
     * @param resolveType A GraphQLTypeResolver for the interface
     */
    createInterface(spec: string, resolveType: GraphQLTypeResolver<mixed, mixed>): SchemaFactory;
    /**
     * Add a single enum type to the schema.
     * @param spec A single type declaration as GraphQL schema IDL
     */
    createEnum(spec: string): SchemaFactory;
    /**
     * Add a single union type to the schema.
     * @param spec A single type declaration as GraphQL schema IDL
     * @param resolveType A GraphQLTypeResolver for the union
     */
    createUnion(spec: string, resolveType: GraphQLTypeResolver<mixed, mixed>): SchemaFactory;
    /**
     * Extends the factories type cache with prebuilt GraphQL types.
     * @param types
     */
    extendWithTypes(types: Array<GraphQLNamedType>): SchemaFactory;
    /**
     * Append a GraphQL IDL document to the factory. Any type collisions are resolved
     * via the schema's CollisionResolver.
     *
     * @param spec A GraphQL document string containing the new schema elements
     */
    extendWithSpec(spec: string, resolvers?: TypeResolverMap<mixed, mixed>): SchemaFactory;
    /**
     * Checks both the type cache and node cache for the existence of a type. If that type exists error.
     * @param name The name of the type.
     */
    private assertIsNewType(name);
    /**
     * Protected API
     * @param directiveNode
     */
    protected getDirective(directiveNode: DirectiveDefinitionNode): GraphQLDirective;
    protected getObjectTypeNamed(name: string): GraphQLObjectType;
    protected getObjectType(typeNode: TypeDefinitionNode): GraphQLObjectType;
    protected produceType(typeNode: TypeNode): GraphQLType;
    protected produceInputType(typeNode: TypeNode): GraphQLInputType;
    protected produceOutputType(typeNode: TypeNode): GraphQLOutputType;
    protected produceObjectType(typeNode: TypeNode): GraphQLObjectType;
    protected produceInterfaceType(typeNode: TypeNode): GraphQLInterfaceType;
    protected produceDirectiveValue(directiveNode: DirectiveNode): GraphQLDirectiveValue;
    protected typeDefNamed(typeName: string): GraphQLNamedType;
    protected makeSchemaDef(def: TypeDefinitionNode): GraphQLObjectType | GraphQLInterfaceType | GraphQLEnumType | GraphQLUnionType | GraphQLInputObjectType | GraphQLScalarType;
    protected makeTypeDef(def: ObjectTypeDefinitionNode): GraphQLObjectType;
    protected getResolver(type: ObjectTypeDefinitionNode, field: FieldDefinitionNode): GraphQLFieldResolver<mixed, mixed> | undefined;
    protected makeObjectFieldDefMap(def: ObjectTypeDefinitionNode): GraphQLFieldConfigMapExt<mixed, mixed>;
    protected makeInterfaceFieldDefMap(def: InterfaceTypeDefinitionNode): GraphQLFieldConfigMapExt<mixed, mixed>;
    protected makeImplementedInterfaces(def: ObjectTypeDefinitionNode): Array<GraphQLInterfaceType>;
    protected makeDirectiveValues(def: ObjectTypeDefinitionNode | FieldDefinitionNode): Array<GraphQLDirectiveValue>;
    protected makeInputValues(values: Array<InputValueDefinitionNode>): {
        [name: string]: mixed;
    };
    protected makeInterfaceDef(def: InterfaceTypeDefinitionNode): GraphQLInterfaceType;
    protected makeEnumDef(def: EnumTypeDefinitionNode): GraphQLEnumType;
    protected makeUnionDef(def: UnionTypeDefinitionNode): GraphQLUnionType;
    protected makeScalarDef(def: ScalarTypeDefinitionNode): GraphQLScalarType;
    protected makeInputObjectDef(def: InputObjectTypeDefinitionNode): GraphQLInputObjectType;
}
